/*************************************************
* DER Encoder Header File                        *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_DER_ENCODER_H__
#define BOTAN_DER_ENCODER_H__

#include <botan/asn1_oid.h>
#include <botan/bigint.h>

namespace Botan {

/*************************************************
* General DER Encoding Object                    *
*************************************************/
class DER_Encoder
   {
   public:
      SecureVector<byte> get_contents();

      void start_sequence(ASN1_Tag = SEQUENCE);
      void end_sequence(ASN1_Tag = SEQUENCE);

      void add_raw_octets(const SecureVector<byte>&);
      void add_object(ASN1_Tag, const byte[], u32bit);
      void add_object(ASN1_Tag, byte);

      DER_Encoder();
   private:
      class DER_Sequence
         {
         public:
            ASN1_Tag tag_of() const { return tag; }
            SecureVector<byte> get_contents();
            void add_bytes(const byte[], u32bit);
            DER_Sequence(ASN1_Tag);
         private:
            ASN1_Tag tag;
            SecureVector<byte> contents;
         };
      SecureVector<byte> contents;
      std::vector<DER_Sequence> subsequences;
      u32bit sequence_level;
   };

/*************************************************
* DER Encoding Functions                         *
*************************************************/
namespace DER {

void encode_null(DER_Encoder&);
void encode(DER_Encoder&, bool);
void encode(DER_Encoder&, u32bit);
void encode(DER_Encoder&, const BigInt&);
void encode(DER_Encoder&, const SecureVector<byte>&, ASN1_Tag);
void encode(DER_Encoder&, const byte[], u32bit, ASN1_Tag);
void encode(DER_Encoder&, const OID&);

}

}

#endif
