/*************************************************
* Common ASN.1 Objects Source File               *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/asn1_obj.h>

namespace Botan {

/*************************************************
* Create an AlgorithmIdentifier                  *
*************************************************/
AlgorithmIdentifier::AlgorithmIdentifier(const X509_PublicKey& key)
   {
   oid = key.get_oid();
   parameters = key.DER_encode_params();
   }

/*************************************************
* Create an AlgorithmIdentifier                  *
*************************************************/
AlgorithmIdentifier::AlgorithmIdentifier(const OID& o,
                                         const SecureVector<byte>& p) :
   oid(o), parameters(p)
   {
   }

namespace DER {

/*************************************************
* DER encode an AlgorithmIdentifier              *
*************************************************/
void encode(DER_Encoder& encoder, const AlgorithmIdentifier& alg_id)
   {
   encoder.start_sequence();
   DER::encode(encoder, alg_id.oid);
   encoder.add_raw_octets(alg_id.parameters);
   encoder.end_sequence();
   }

}

namespace BER {

/*************************************************
* Decode a BER encoded AlgorithmIdentifier       *
*************************************************/
void decode(BER_Decoder& source, AlgorithmIdentifier& alg_id)
   {
   BER_Decoder sequence = BER::get_subsequence(source);
   BER::decode(sequence, alg_id.oid);
   alg_id.parameters = sequence.get_remaining();
   sequence.verify_end();
   }

}

}
