/*************************************************
* ASN.1 OID Source File                          *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/asn1_oid.h>
#include <botan/util.h>

namespace Botan {

/*************************************************
* Return this OID as a string                    *
*************************************************/
std::string OID::as_string() const
   {
   std::string oid_str;
   for(u32bit j = 0; j != id.size(); j++)
      {
      oid_str += to_string(id[j]);
      if(j != id.size() - 1)
         oid_str += '.';
      }
   return oid_str;
   }

/*************************************************
* OID equality comparison                        *
*************************************************/
bool OID::operator==(const OID& oid) const
   {
   if(id.size() != oid.id.size())
      return false;
   for(u32bit j = 0; j != id.size(); j++)
      if(id[j] != oid.id[j])
         return false;
   return true;
   }

/*************************************************
* Append another component to the OID            *
*************************************************/
OID& OID::operator+=(u32bit component)
   {
   id.push_back(component);
   return (*this);
   }

/*************************************************
* ASN.1 OID Constructor                          *
*************************************************/
OID::OID(const std::string& oid_str)
   {
   if(oid_str != "")
      {
      id = parse_asn1_oid(oid_str);
      if(id.size() < 2 || id[0] > 2)
         throw Invalid_OID(oid_str);
      if((id[0] == 0 || id[0] == 1) && id[1] > 39)
         throw Invalid_OID(oid_str);
      }
   }

/*************************************************
* Append another component to the OID            *
*************************************************/
OID operator+(const OID& oid, u32bit component)
   {
   OID new_oid(oid);
   new_oid += component;
   return new_oid;
   }

/*************************************************
* OID inequality comparison                      *
*************************************************/
bool operator!=(const OID& a, const OID& b)
   {
   return !(a == b);
   }

/*************************************************
* Compare two OIDs                               *
*************************************************/
bool operator<(const OID& a, const OID& b)
   {
   std::vector<u32bit> oid1 = a.get_id();
   std::vector<u32bit> oid2 = b.get_id();

   if(oid1.size() < oid2.size())
      return true;
   if(oid1.size() > oid2.size())
      return false;
   for(u32bit j = 0; j != oid1.size(); j++)
      {
      if(oid1[j] < oid2[j])
         return true;
      if(oid1[j] > oid2[j])
         return false;
      }
   return false;
   }

}
