/*************************************************
* OFB Mode Source File                           *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/ofb.h>

namespace Botan {

/*************************************************
* OFB Constructor                                *
*************************************************/
OFB::OFB(const std::string& cipher_name, const SymmetricKey& key,
         const InitializationVector& iv) :
   BlockCipherMode(cipher_name, "OFB", key, iv)
   {
   if(!valid_iv_length(iv.length()))
      throw Invalid_IV_Length(name(), iv.length());
   cipher->encrypt(state);
   }

/*************************************************
* OFB Encryption/Decryption                      *
*************************************************/
void OFB::write(const byte input[], u32bit length)
   {
   while(length)
      {
      u32bit copied = std::min(BLOCK_SIZE - position, length);
      xor_buf(buffer, input, state + position, copied);
      send(buffer, copied);
      input += copied;
      length -= copied;
      position += copied;
      if(position == BLOCK_SIZE)
         {
         cipher->encrypt(state);
         position = 0;
         }
      }
   }

}
