/*************************************************
* Global RNG Seeding Source File                 *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/rng.h>
#include <botan/es_file.h>

#if defined(BOTAN_EXT_ENTROPY_SRC_UNIX)
  #include <botan/es_unix.h>
#endif

#if defined(BOTAN_EXT_ENTROPY_SRC_FTW)
  #include <botan/es_ftw.h>
#endif

#if defined(BOTAN_EXT_ENTROPY_SRC_EGD)
  #include <botan/es_egd.h>
#endif

namespace Botan {

namespace {

/*************************************************
* Try to do a poll on an EntropySource           *
*************************************************/
u32bit poll_es(EntropySource& es, u32bit buf_size = 256)
   {
   SecureVector<byte> buffer(buf_size);
   u32bit got = es.slow_poll(buffer, buffer.size());
   Global_RNG::add_entropy(buffer, got);
   return entropy_estimate(buffer, got);
   }

}

namespace Global_RNG {

/*************************************************
* Seed the global RNG                            *
*************************************************/
bool seed(u32bit level, const std::string& egd_path)
   {
   static const u32bit MIN_ENTROPY = 256;

   if(level > 2)
      level = 2;

   u32bit bits = 0;
   File_EntropySource es_file;
   bits += poll_es(es_file, 512);

   if(level == 0 || ((bits >= MIN_ENTROPY) && level == 1))
      return (bits >= MIN_ENTROPY);

   if(egd_path != "")
      {
#if defined(BOTAN_EXT_ENTROPY_SRC_EGD)
      EGD_EntropySource es_egd(egd_path);
      bits += poll_es(es_egd);
#endif
      }

   if(level == 1 || ((bits >= MIN_ENTROPY) && level == 2))
      return (bits >= MIN_ENTROPY);

#if defined(BOTAN_EXT_ENTROPY_SRC_UNIX)
   Unix_EntropySource es_unix;
   bits += poll_es(es_unix);
#endif

#if defined(BOTAN_EXT_ENTROPY_SRC_FTW)
   FTW_EntropySource es_ftw;
   bits += poll_es(es_ftw);
#endif

   return (bits >= MIN_ENTROPY);
   }

}

}
